@@ -24,17 +24,19 @@
 /**
  * SECTION:element-wavpackparse
  *
+ * <refsect2>
  * WavpackParse takes raw, unframed Wavpack streams and splits them into
  * single Wavpack chunks with information like bit depth and the position
  * in the stream.
  * <ulink url="http://www.wavpack.com/">Wavpack</ulink> is an open-source
  * audio codec that features both lossless and lossy encoding.
- *
- * <refsect2>
  * <title>Example launch line</title>
- * |[
+ * <para>
+ * <programlisting>
  * gst-launch filesrc location=test.wv ! wavpackparse ! wavpackdec ! fakesink
- * ]| This pipeline decodes the Wavpack file test.wv into raw audio buffers.
+ * </programlisting>
+ * This pipeline decodes the Wavpack file test.wv into raw audio buffers.
+ * </para>
  * </refsect2>
  */
 
@@ -56,7 +58,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_wavpack_parse_debug);
 #define GST_CAT_DEFAULT gst_wavpack_parse_debug
 
 static inline GstWavpackParseIndexEntry *
-gst_wavpack_parse_index_entry_new (void)
+gst_wavpack_parse_index_entry_new ()
 {
   return g_slice_new (GstWavpackParseIndexEntry);
 }
@@ -91,18 +93,14 @@ static GstStaticPadTemplate wvc_src_factory = GST_STATIC_PAD_TEMPLATE ("wvcsrc",
     );
 
 static gboolean gst_wavpack_parse_sink_activate (GstPad * sinkpad);
-
 static gboolean
 gst_wavpack_parse_sink_activate_pull (GstPad * sinkpad, gboolean active);
 
 static void gst_wavpack_parse_loop (GstElement * element);
-
 static GstStateChangeReturn gst_wavpack_parse_change_state (GstElement *
     element, GstStateChange transition);
 static void gst_wavpack_parse_reset (GstWavpackParse * parse);
-
 static gint64 gst_wavpack_parse_get_upstream_length (GstWavpackParse * wvparse);
-
 static GstBuffer *gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse,
     gint64 offset, guint size, GstFlowReturn * flow);
 static GstFlowReturn gst_wavpack_parse_chain (GstPad * pad, GstBuffer * buf);
@@ -115,10 +113,12 @@ gst_wavpack_parse_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &wvc_src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&wvc_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
   gst_element_class_set_details_simple (element_class, "Wavpack parser",
       "Codec/Demuxer/Audio",
@@ -139,13 +139,12 @@ static void
 gst_wavpack_parse_class_init (GstWavpackParseClass * klass)
 {
   GObjectClass *gobject_class;
-
   GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = gst_wavpack_parse_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_wavpack_parse_finalize);
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_wavpack_parse_change_state);
 }
@@ -163,7 +162,6 @@ gst_wavpack_parse_index_get_entry_from_sample (GstWavpackParse * wvparse,
     gint64 sample_offset)
 {
   gint i;
-
   GSList *node;
 
   if (wvparse->entries == NULL)
@@ -222,7 +220,7 @@ gst_wavpack_parse_index_append_entry (GstWavpackParse * wvparse,
 static void
 gst_wavpack_parse_reset (GstWavpackParse * parse)
 {
-  parse->total_samples = G_GINT64_CONSTANT (-1);
+  parse->total_samples = -1;
   parse->samplerate = 0;
   parse->channels = 0;
 
@@ -284,15 +282,12 @@ static gboolean
 gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
 {
   GstWavpackParse *parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
-
   GstFormat format;
-
   gboolean ret = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:{
       gint64 cur;
-
       guint rate;
 
       GST_OBJECT_LOCK (parse);
@@ -327,7 +322,6 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
     }
     case GST_QUERY_DURATION:{
       gint64 len;
-
       guint rate;
 
       GST_OBJECT_LOCK (parse);
@@ -344,7 +338,7 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
 
       switch (format) {
         case GST_FORMAT_TIME:
-          if (len != G_GINT64_CONSTANT (-1))
+          if (len != -1)
             len = gst_util_uint64_scale_int (len, GST_SECOND, rate);
           gst_query_set_duration (query, GST_FORMAT_TIME, len);
           ret = TRUE;
@@ -365,7 +359,6 @@ gst_wavpack_parse_src_query (GstPad * pad, GstQuery * query)
       gst_query_parse_seeking (query, &format, NULL, NULL, NULL);
       if (format == GST_FORMAT_TIME || format == GST_FORMAT_DEFAULT) {
         gboolean seekable;
-
         gint64 duration = -1;
 
         /* only fails if we didn't read the headers yet and can't say
@@ -406,9 +399,7 @@ gst_wavpack_parse_scan_to_find_sample (GstWavpackParse * parse,
     gint64 sample, gint64 * byte_offset, gint64 * start_sample)
 {
   GstWavpackParseIndexEntry *entry;
-
   GstFlowReturn ret;
-
   gint64 off = 0;
 
   /* first, check if we have to scan at all */
@@ -436,7 +427,6 @@ gst_wavpack_parse_scan_to_find_sample (GstWavpackParse * parse,
   /* now scan forward until we find the chunk we're looking for or hit EOS */
   do {
     WavpackHeader header;
-
     GstBuffer *buf;
 
     buf = gst_wavpack_parse_pull_buffer (parse, off, sizeof (WavpackHeader),
@@ -474,15 +464,10 @@ static gboolean
 gst_wavpack_parse_send_newsegment (GstWavpackParse * wvparse, gboolean update)
 {
   GstSegment *s = &wvparse->segment;
-
   gboolean ret;
-
   gint64 stop_time = -1;
-
   gint64 start_time = 0;
-
   gint64 cur_pos_time;
-
   gint64 diff;
 
   /* segment is in DEFAULT format, but we want to send a TIME newsegment */
@@ -519,31 +504,18 @@ gst_wavpack_parse_handle_seek_event (GstWavpackParse * wvparse,
     GstEvent * event)
 {
   GstSeekFlags seek_flags;
-
   GstSeekType start_type;
-
   GstSeekType stop_type;
-
   GstSegment segment;
-
   GstFormat format;
-
   gboolean only_update;
-
   gboolean flush, ret;
-
   gdouble speed;
-
   gint64 stop;
-
   gint64 start;                 /* sample we want to seek to                  */
-
   gint64 byte_offset;           /* byte offset the chunk we seek to starts at */
-
   gint64 chunk_start;           /* first sample in chunk we seek to           */
-
   guint rate;
-
   gint64 last_stop;
 
   if (wvparse->adapter) {
@@ -669,7 +641,6 @@ static gboolean
 gst_wavpack_parse_sink_event (GstPad * pad, GstEvent * event)
 {
   GstWavpackParse *parse;
-
   gboolean ret = TRUE;
 
   parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
@@ -728,7 +699,6 @@ static gboolean
 gst_wavpack_parse_src_event (GstPad * pad, GstEvent * event)
 {
   GstWavpackParse *parse;
-
   gboolean ret;
 
   parse = GST_WAVPACK_PARSE (gst_pad_get_parent (pad));
@@ -750,7 +720,6 @@ static void
 gst_wavpack_parse_init (GstWavpackParse * parse, GstWavpackParseClass * gclass)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (parse);
-
   GstPadTemplate *tmpl;
 
   tmpl = gst_element_class_get_pad_template (klass, "sink");
@@ -791,12 +760,11 @@ gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse, gint64 offset,
     guint size, GstFlowReturn * flow)
 {
   GstFlowReturn flow_ret;
-
   GstBuffer *buf = NULL;
 
-  if (offset + size > wvparse->upstream_length) {
+  if (offset + size >= wvparse->upstream_length) {
     wvparse->upstream_length = gst_wavpack_parse_get_upstream_length (wvparse);
-    if (offset + size > wvparse->upstream_length) {
+    if (offset + size >= wvparse->upstream_length) {
       GST_DEBUG_OBJECT (wvparse, "EOS: %" G_GINT64_FORMAT " + %u > %"
           G_GINT64_FORMAT, offset, size, wvparse->upstream_length);
       flow_ret = GST_FLOW_UNEXPECTED;
@@ -832,9 +800,7 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
   GstWavpackMetadata meta;
-
   GstCaps *caps = NULL;
-
   guchar *bufptr;
 
   g_assert (wvparse->srcpad == NULL);
@@ -854,13 +820,9 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
       case ID_WV_BITSTREAM:
       case ID_WVX_BITSTREAM:{
         WavpackStreamReader *stream_reader = gst_wavpack_stream_reader_new ();
-
         WavpackContext *wpc;
-
         gchar error_msg[80];
-
         read_id rid;
-
         gint channel_mask;
 
         rid.buffer = GST_BUFFER_DATA (buf);
@@ -875,9 +837,8 @@ gst_wavpack_parse_create_src_pad (GstWavpackParse * wvparse, GstBuffer * buf,
 
         wvparse->samplerate = WavpackGetSampleRate (wpc);
         wvparse->channels = WavpackGetNumChannels (wpc);
-        wvparse->total_samples =
-            (header->total_samples ==
-            0xffffffff) ? G_GINT64_CONSTANT (-1) : header->total_samples;
+        wvparse->total_samples = (header->total_samples == (int32_t) - 1) ?
+            -1 : header->total_samples;
 
         caps = gst_caps_new_simple ("audio/x-wavpack",
             "width", G_TYPE_INT, WavpackGetBitsPerSample (wpc),
@@ -948,7 +909,6 @@ static GstFlowReturn
 gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
-  GstFlowReturn ret;
   wvparse->current_offset += header->ckSize + 8;
 
   wvparse->segment.last_stop = header->block_index;
@@ -1006,18 +966,13 @@ gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
   GST_LOG_OBJECT (wvparse, "Pushing buffer with time %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
-  ret = gst_pad_push (wvparse->srcpad, buf);
-
-  wvparse->segment.last_stop = wvparse->next_block_index;
-
-  return ret;
+  return gst_pad_push (wvparse->srcpad, buf);
 }
 
 static guint8 *
 gst_wavpack_parse_find_marker (guint8 * buf, guint size)
 {
   int i;
-
   guint8 *ret = NULL;
 
   if (G_UNLIKELY (size < 4))
@@ -1036,13 +991,11 @@ static GstFlowReturn
 gst_wavpack_parse_resync_loop (GstWavpackParse * parse, WavpackHeader * header)
 {
   GstFlowReturn flow_ret = GST_FLOW_UNEXPECTED;
-
   GstBuffer *buf = NULL;
 
   /* loop until we have a frame header or reach the end of the stream */
   while (1) {
     guint8 *data, *marker;
-
     guint len, size;
 
     if (buf) {
@@ -1103,7 +1056,6 @@ static void
 gst_wavpack_parse_loop (GstElement * element)
 {
   GstWavpackParse *parse = GST_WAVPACK_PARSE (element);
-
   GstFlowReturn flow_ret;
   WavpackHeader header = { {0,}, 0, };
   GstBuffer *buf = NULL;
@@ -1146,29 +1098,30 @@ pause:
     GST_LOG_OBJECT (parse, "pausing task, reason %s", reason);
     gst_pad_pause_task (parse->sinkpad);
 
-    if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
-      if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        GstClockTime stop;
+    if (GST_FLOW_IS_FATAL (flow_ret) || flow_ret == GST_FLOW_NOT_LINKED) {
+      if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
+        if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+          GstClockTime stop;
 
-        GST_LOG_OBJECT (parse, "Sending segment done");
+          GST_LOG_OBJECT (parse, "Sending segment done");
 
-        if ((stop = parse->segment.stop) == -1)
-          stop = parse->segment.duration;
+          if ((stop = parse->segment.stop) == -1)
+            stop = parse->segment.duration;
 
-        gst_element_post_message (GST_ELEMENT_CAST (parse),
-            gst_message_new_segment_done (GST_OBJECT_CAST (parse),
-                parse->segment.format, stop));
+          gst_element_post_message (GST_ELEMENT_CAST (parse),
+              gst_message_new_segment_done (GST_OBJECT_CAST (parse),
+                  parse->segment.format, stop));
+        } else {
+          GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
+          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
+        }
       } else {
-        GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
-        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
+        GST_ELEMENT_ERROR (parse, STREAM, FAILED,
+            (_("Internal data stream error.")),
+            ("stream stopped, reason %s", reason));
+        if (parse->srcpad)
+          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
       }
-    } else if (flow_ret == GST_FLOW_NOT_LINKED
-        || flow_ret < GST_FLOW_UNEXPECTED) {
-      GST_ELEMENT_ERROR (parse, STREAM, FAILED,
-          (_("Internal data stream error.")), ("stream stopped, reason %s",
-              reason));
-      if (parse->srcpad)
-        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
     }
     return;
   }
@@ -1178,7 +1131,6 @@ static gboolean
 gst_wavpack_parse_resync_adapter (GstAdapter * adapter)
 {
   const guint8 *buf, *marker;
-
   guint avail = gst_adapter_available (adapter);
 
   if (avail < 4)
@@ -1210,11 +1162,8 @@ static GstFlowReturn
 gst_wavpack_parse_chain (GstPad * pad, GstBuffer * buf)
 {
   GstWavpackParse *wvparse = GST_WAVPACK_PARSE (GST_PAD_PARENT (pad));
-
   GstFlowReturn ret = GST_FLOW_OK;
-
   WavpackHeader wph;
-
   const guint8 *tmp_buf;
 
   if (!wvparse->adapter) {
@@ -1274,7 +1223,6 @@ static GstStateChangeReturn
 gst_wavpack_parse_change_state (GstElement * element, GstStateChange transition)
 {
   GstWavpackParse *wvparse = GST_WAVPACK_PARSE (element);
-
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   switch (transition) {
