@@ -29,6 +29,10 @@
 #include "audioinvert.h"
 #include "audioamplify.h"
 #include "audiodynamic.h"
+#include "audiocheblimit.h"
+#include "audiochebband.h"
+#include "audiowsincband.h"
+#include "audiowsinclimit.h"
 
 /* entry point to initialize the plug-in
  * initialize the plug-in itself
@@ -48,11 +52,19 @@ plugin_init (GstPlugin * plugin)
       gst_element_register (plugin, "audioamplify", GST_RANK_NONE,
           GST_TYPE_AUDIO_AMPLIFY) &&
       gst_element_register (plugin, "audiodynamic", GST_RANK_NONE,
-          GST_TYPE_AUDIO_DYNAMIC));
+          GST_TYPE_AUDIO_DYNAMIC) &&
+      gst_element_register (plugin, "audiocheblimit", GST_RANK_NONE,
+          GST_TYPE_AUDIO_CHEB_LIMIT) &&
+      gst_element_register (plugin, "audiochebband", GST_RANK_NONE,
+          GST_TYPE_AUDIO_CHEB_BAND) &&
+      gst_element_register (plugin, "audiowsinclimit", GST_RANK_NONE,
+          GST_TYPE_AUDIO_WSINC_LIMIT) &&
+      gst_element_register (plugin, "audiowsincband", GST_RANK_NONE,
+          GST_TYPE_AUDIO_WSINC_BAND));
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "audiofx",
     "Audio effects plugin",
-    plugin_init, VERSION, "LGPL", "GStreamer", "http://gstreamer.net/")
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
