@@ -77,24 +77,16 @@ GST_STATIC_PAD_TEMPLATE ("src",
         /* "auxiliarydatasizelength = (string) [0, 64]" */ )
     );
 
-enum
-{
-  PROP_0,
-};
-
 
 static void gst_rtp_mp4g_pay_class_init (GstRtpMP4GPayClass * klass);
 static void gst_rtp_mp4g_pay_base_init (GstRtpMP4GPayClass * klass);
 static void gst_rtp_mp4g_pay_init (GstRtpMP4GPay * rtpmp4gpay);
 static void gst_rtp_mp4g_pay_finalize (GObject * object);
 
-static void gst_rtp_mp4g_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp4g_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static gboolean gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_mp4g_pay_change_state (GstElement * element,
+    GstStateChange transition);
 static GstFlowReturn gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload *
     payload, GstBuffer * buffer);
 
@@ -151,11 +143,10 @@ gst_rtp_mp4g_pay_class_init (GstRtpMP4GPayClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->set_property = gst_rtp_mp4g_pay_set_property;
-  gobject_class->get_property = gst_rtp_mp4g_pay_get_property;
-
   gobject_class->finalize = gst_rtp_mp4g_pay_finalize;
 
+  gstelement_class->change_state = gst_rtp_mp4g_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_mp4g_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_mp4g_pay_handle_buffer;
 
@@ -184,6 +175,10 @@ gst_rtp_mp4g_pay_finalize (GObject * object)
   g_free (rtpmp4gpay->params);
   rtpmp4gpay->params = NULL;
 
+  if (rtpmp4gpay->config)
+    gst_buffer_unref (rtpmp4gpay->config);
+  rtpmp4gpay->config = NULL;
+
   g_free (rtpmp4gpay->profile);
   rtpmp4gpay->profile = NULL;
 
@@ -517,7 +512,8 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
     /* marker only if the packet is complete */
     gst_rtp_buffer_set_marker (outbuf, avail <= payload_len);
 
-    GST_BUFFER_TIMESTAMP (outbuf) = rtpmp4gpay->first_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = rtpmp4gpay->first_timestamp;
+    GST_BUFFER_DURATION (outbuf) = rtpmp4gpay->first_duration;
 
     ret = gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtpmp4gpay), outbuf);
 
@@ -541,7 +537,8 @@ gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   rtpmp4gpay = GST_RTP_MP4G_PAY (basepayload);
 
-  rtpmp4gpay->first_ts = GST_BUFFER_TIMESTAMP (buffer);
+  rtpmp4gpay->first_timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  rtpmp4gpay->first_duration = GST_BUFFER_DURATION (buffer);
 
   /* we always encode and flush a full AU */
   gst_adapter_push (rtpmp4gpay->adapter, buffer);
@@ -550,34 +547,29 @@ gst_rtp_mp4g_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   return ret;
 }
 
-static void
-gst_rtp_mp4g_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
+static GstStateChangeReturn
+gst_rtp_mp4g_pay_change_state (GstElement * element, GstStateChange transition)
 {
   GstRtpMP4GPay *rtpmp4gpay;
+  GstStateChangeReturn ret;
 
-  rtpmp4gpay = GST_RTP_MP4G_PAY (object);
+  rtpmp4gpay = GST_RTP_MP4G_PAY (element);
 
-  switch (prop_id) {
+  switch (transition) {
     default:
       break;
   }
-}
 
-static void
-gst_rtp_mp4g_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4GPay *rtpmp4gpay;
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
-  rtpmp4gpay = GST_RTP_MP4G_PAY (object);
-
-  switch (prop_id) {
+  switch (transition) {
     default:
       break;
   }
+  return ret;
 }
 
+
 gboolean
 gst_rtp_mp4g_pay_plugin_init (GstPlugin * plugin)
 {
